import sys
import traceback as tb


class Report(object):
    """
    The base report.

    Attributes:
        succeeded (bool): Indicates whether or not a handler operation succeeded.
        details (dict): Operation result details.
        num_changes (int): The number of changes made during the operation.
        The granularity is up to the discretion of the handler.
    """

    def __init__(self):
        self.succeeded = True
        self.details = {}
        self.num_changes = 0

    def dict(self):
        """
        Dictionary representation of the report.

        Returns:
            dict: Self.
        """
        return self.__dict__

    def __str__(self):
        return str(self.dict())

    def __len__(self):
        return self.num_changes


class HandlerReport(Report):
    """
    The base handler report.

    Attributes:
        aggregation_key (str): The key used to aggregate details when
            updating another report.
    """

    @staticmethod
    def last_exception():
        """
        Get details of the last raised exception.

        Returns:
            dict: Details of last raised exception.
        """
        details = {}
        info = sys.exc_info()
        inst = info[1]
        trace = '\n'.join(tb.format_exception(*info))
        details['message'] = str(inst)
        details['trace'] = trace
        return details

    def __init__(self):
        Report.__init__(self)
        self.aggregation_key = None

    def set_succeeded(self, details=None, num_changes=0):
        """
        Called to indicate an operation succeeded.

        Args:
            details (dict): The details of the operation result.
            num_changes (int): The number of changes made during the operation.
                The granularity is up to the discretion of the handler.
        """
        self.succeeded = True
        self.details = (details or {})
        self.num_changes = num_changes

    def set_failed(self, details=None):
        """
        Called to indicate an operation failed.

        Args:
            details (dict): The details of the operation result.
        """
        self.succeeded = False
        self.details = (details or {})

    def update(self, report):
        """
        Update the specified report.

        Args:
            report (DispatchReport): A dispatch report.
        """
        self._update_succeeded(report)
        self._update_num_changes(report)
        self._update_details(report)

    def _update_succeeded(self, report):
        """
        Update the succeeded flag in the specified report.

        Args:
            report (DispatchReport): A dispatch report.
        """
        if not self.succeeded:
            report.succeeded = self.succeeded

    def _update_num_changes(self, report):
        """
        Update the num_changes in the specified report.

        Args:
            report (DispatchReport): A dispatch report.
        """
        if self.succeeded:
            report.num_changes += self.num_changes

    def _update_details(self, report):
        """
        Update the details in the specified report.
        The details are aggregated in the specified report using
        the aggregation key.

        Args:
            report (DispatchReport): A dispatch report.
        """
        report.details[self.aggregation_key] = dict(
            succeeded=self.succeeded,
            details=self.details)


class ContentReport(HandlerReport):
    """
    The content report is returned by handler methods
    implementing content unit operations.
    """
    pass


class DispatchReport(Report):
    """
    The (internal) dispatch report is returned for all handler methods
    dispatched to handlers.  It represents an aggregation of handler reports.
    The handler (class) reports are collated by type_id (content, distributor, system).
    The overall succeeded is True (succeeded) only if all of the handler reports have
    a succeeded of True (succeeded).
    Succeeded Example:
      { 'succeeded' : True,
        'num_changes' : 10,
        'reboot' : { 'scheduled' : False, details : {} },
        'details' : {
          'type_A' : { 'succeeded' : True, 'details' : {} },
          'type_B' : { 'succeeded' : True, 'details' : {} },
          'type_C' : { 'succeeded' : True, 'details' : {} },
        }
      }
    Failed Example:
      { 'succeeded' : False,
        'num_changes' : 6,
        'reboot' : { 'scheduled' : False, details : {} },
        'details' : {
          'type_A' : { 'succeeded' : True, 'details' : {} },
          'type_B' : { 'succeeded' : True, 'details' : {} },
          'type_C' : { 'succeeded' : False,
                       'details' : { 'message' : <message>, 'trace'=<trace> } },
        }
      }

    Attributes:
        succeeded (bool): The overall succeeded (succeeded|failed).
        details (dict): operation details keyed by aggregation_key.
            Each value is:
                { 'succeeded' : True, details : {} }
        num_changes (int): The number of changes made during the operation.
            The granularity is up to the discretion of the handlers.
    """

    def __init__(self):
        Report.__init__(self)
        self.reboot = dict(scheduled=False, details={})



import six
import logging

from itertools import chain
from gettext import gettext as _
from optparse import OptionParser

from yum import YumBase
from yum.plugins import TYPE_CORE, TYPE_INTERACTIVE
from yum.Errors import InstallError
from yum import constants
from yum import updateinfo


log = logging.getLogger(__name__)
logfile = logging.getLogger('yum.filelogging')


class Pattern(object):
    """
    Package matching pattern.

    Attributes:
        name (str): Name.
        epoch (str): Epoch.
        version (str): Version.
        release (str): Release.
        arch (str) Arch..
    """

    __slots__ = (
        'name',
        'epoch',
        'version',
        'release',
        'arch',
    )

    FIELDS = (
        ('epoch', ('', ':')),
        ('name', ('', '')),
        ('version', ('-', '')),
        ('release', ('-', '')),
        ('arch', ('.', '')),
    )

    def __init__(self, name, epoch='', version='', release='', arch=''):
        """
        NEVRA

        Args:
            name (str): Name.
            epoch (str): Epoch.
            version (str): Version.
            release (str): Release.
            arch (str) Arch.
        """
        self.name = name
        self.epoch = epoch
        self.version = version
        self.release = release
        self.arch = arch

    def __eq__(self, other):
       return str(self) == str(other)

    def __str__(self):
        pattern = []
        for name, sep in self.FIELDS:
            value = getattr(self, name)
            if not value:
                continue
            if sep[0]:
                pattern.append(sep[0])
            pattern.append(value)
            if sep[1]:
                pattern.append(sep[1])
        return ''.join(pattern)


class TransactionReport(object):
    """
    YUM transaction report.
    """

    def __init__(self, ts_info, states):
        """
        Args:
            ts_info: A YUM transaction.
            states: List of package states to be included.
        """
        self.deps = []
        self.resolved = []
        self.failed = []
        for t in ts_info:
            if t.output_state not in states:
                continue
            qname = str(t.po)
            package = dict(
                qname=qname,
                repoid=t.repoid,
                name=t.po.name,
                version=t.po.ver,
                release=t.po.rel,
                arch=t.po.arch,
                epoch=t.po.epoch)
            if t.output_state == constants.TS_FAILED:
                self.failed.append(package)
            if t.isDep:
                self.deps.append(package)
            else:
                self.resolved.append(package)

    def dict(self):
        """
        Dictionary representation.

        Returns:
            dict: self
        """
        return {
            'resolved': self.resolved,
            'failed': self.failed,
            'deps': self.deps,
        }

    def log(self, heading):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        if not self:
            return
        for qname in [p['qname'] for p in chain(self.resolved, self.deps)]:
            entry = heading.format(qname)
            logfile.info(entry)
            log.info(entry)

    def __len__(self):
        return len(self.resolved) + len(self.deps)


class InstallTxReport(TransactionReport):
    """
    Install transaction report.
    """

    HEADING = _('Installed: {}')

    STATES = (
        constants.TS_FAILED,
        constants.TS_INSTALL,
        constants.TS_TRUEINSTALL,
        constants.TS_UPDATE
    )

    def __init__(self, ts_info, states=STATES):
        """
        Args:
            ts_info: A YUM transaction.
            states: List of package states to be included.
        """
        super(InstallTxReport, self).__init__(ts_info, states)

    def log(self, heading=HEADING):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        super(InstallTxReport, self).log(heading)


class UpdateTxReport(TransactionReport):
    """
    Update transaction report.
    """

    HEADING = _('Updated: {}')

    STATES = (
        constants.TS_FAILED,
        constants.TS_INSTALL,
        constants.TS_TRUEINSTALL,
        constants.TS_UPDATE
    )

    def __init__(self, ts_info, states=STATES):
        """
        Args:
            ts_info: A YUM transaction.
            states: List of package states to be included.
        """
        super(UpdateTxReport, self).__init__(ts_info, states)

    def log(self, heading=HEADING):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        super(UpdateTxReport, self).log(heading)


class EraseTxReport(TransactionReport):
    """
    Erase transaction report.
    """

    HEADING = _('Erased: {}')

    STATES = (
        constants.TS_FAILED,
        constants.TS_ERASE
    )

    def __init__(self, ts_info, states=STATES):
        """
        Args:
            ts_info: A YUM transaction.
            states: List of package states to be included.
        """
        super(EraseTxReport, self).__init__(ts_info, states)

    def log(self, heading=HEADING):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        super(EraseTxReport, self).log(heading)


class API(object):
    """
    Abstract package management API.

    Attributes:
        commit (bool): Commit the transaction.
    """

    def __init__(self, commit=True):
        """
        Args:
            commit (bool): Commit the transaction.
                Use False for a "dry run".
        """
        self.commit = commit


class Package(API):
    """
    The package management API.
    """

    def install(self, patterns):
        """
        Install packages.

        Args:
            patterns (collections.Iterable): A list of Pattern.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibYum() as lib:
            for pattern in patterns:
                try:
                    lib.install(pattern=str(pattern))
                except InstallError as caught:
                    description = six.text_type(caught).encode('utf-8')
                    caught.value = '%s: %s' % (pattern, description)
                    raise caught
            lib.resolveDeps()
            if self.commit and len(lib.tsInfo):
                lib.processTransaction()
            report = InstallTxReport(lib.tsInfo)
            if self.commit:
                report.log()
            return report.dict()

    def update(self, patterns=(), advisories=()):
        """
        Update packages.

        Args:
            patterns (collections.Iterable): A list of Pattern.
            advisories (collections.Iterable): A list of advisory IDs.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibYum() as lib:
            if advisories:
                lib.updateinfo_filters = {
                    'bzs': [],
                    'bugfix': None,
                    'sevs': [],
                    'security': None,
                    'advs': advisories,
                    'cves': []
                }
                updateinfo.update_minimal(lib)
            else:
                if patterns:
                    for pattern in patterns:
                        lib.update(pattern=str(pattern))
                else:
                    lib.update()
            lib.resolveDeps()
            if self.commit and len(lib.tsInfo):
                lib.processTransaction()
            report = UpdateTxReport(lib.tsInfo)
            if self.commit:
                report.log()
            return report.dict()

    def uninstall(self, patterns):
        """
        Uninstall (remove) packages.

        Args:
            patterns (collections.Iterable): A list of Pattern.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibYum() as lib:
            for pattern in patterns:
                lib.remove(pattern=str(pattern))
            lib.resolveDeps()
            if self.commit and len(lib.tsInfo):
                lib.processTransaction()
            report = EraseTxReport(lib.tsInfo)
            if self.commit:
                report.log()
            return report.dict()


class PackageGroup(API):
    """
    Package group management API.
    """

    def install(self, names):
        """
        Install package groups.

        Args:
            names (collections.Iterable): A list of group names.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibYum() as lib:
            for name in names:
                lib.selectGroup(name)
            lib.resolveDeps()
            if self.commit and len(lib.tsInfo):
                lib.processTransaction()
            report = InstallTxReport(lib.tsInfo)
            if self.commit:
                report.log()
            return report.dict()

    def uninstall(self, names):
        """
        Uninstall package groups.

        Args:
            names (collections.Iterable): A list of group names.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibYum() as lib:
            for name in names:
                lib.groupRemove(name)
            lib.resolveDeps()
            if self.commit and len(lib.tsInfo):
                lib.processTransaction()
            report = EraseTxReport(lib.tsInfo)
            if self.commit:
                report.log()
            return report.dict()


class LibYum(YumBase):
    """
    A YUM base.
    """

    def __init__(self):
        """
        Initialization.
        """
        parser = OptionParser()
        parser.parse_args([])
        self.__parser = parser
        super(LibYum, self).__init__()
        self.preconf.optparser = self.__parser
        self.preconf.plugin_types = (TYPE_CORE, TYPE_INTERACTIVE)
        self.conf.assumeyes = True


    """
    Plugins ordinarily used via CLI may use YumBaseCli to register their respective commands
    Since we extend YumBase we need to make sure any such plugins we've loaded don't raise an error when doing so
    """
    def registerCommand(self, command):
      return self


    def doPluginSetup(self, *args, **kwargs):
        """
        Plugin setup.

        Args:
            *args:
            **kwargs:
        """
        super(LibYum, self).doPluginSetup(*args, **kwargs)
        p = self.__parser
        options, args = p.parse_args([])
        self.plugins.setCmdLine(options, args)

    def __enter__(self):
        return self

    def __exit__(self, *unused):
        self.close()



import socket

from subscription_manager.base_plugin import SubManPlugin

requires_api_version = "1.0"

class FactsPlugin(SubManPlugin):
    name = 'fqdn fact'

    def post_facts_collection_hook(self, conduit):
        if 'network.fqdn' not in conduit.facts:
            conduit.facts['network.fqdn'] = socket.getfqdn()




from __future__ import absolute_import
from os import path

# The path is defined in zypper, see https://github.com/openSUSE/libzypp/blob/master/zypp/target/TargetImpl.cc
REBOOT_NEEDED_FLAG = "/var/run/reboot-needed"


class ZypperTracerApp:
    pass


def collect_apps(plugin=None):
    apps = []
    if path.isfile(REBOOT_NEEDED_FLAG):
        app = ZypperTracerApp()
        app.name = "kernel"
        app.helper = "You will have to reboot your computer"
        app.type = "static"
        apps.append(app)
    return apps



import dnf.cli

from dnfpluginscore import logger

from katello.tracer import query_affected_apps, upload_tracer_profile


class TracerUploadCommand(dnf.cli.Command):
    aliases = ['katello-tracer-upload']
    summary = 'Upload Tracer data to Katello'

    def configure(self):
        self.cli.demands.root_user = True

    def run(self):
        upload_tracer_profile(query_affected_apps, None)


class TracerUpload(dnf.Plugin):
    name = 'tracer-upload'
    config_name = 'tracer_upload'

    def __init__(self, base, cli):
        super(TracerUpload, self).__init__(base, cli)
        if cli:
            cli.register_command(TracerUploadCommand)

    def transaction(self):
        conf = self.read_config(self.base.conf)
        enabled = (conf.has_section('main') and
                   conf.has_option('main', 'enabled') and
                   conf.getboolean('main', 'enabled'))

        if enabled:
            if (conf.has_option('main', 'supress_debug') and not
               conf.getboolean('main', 'supress_debug')):
                logger.info("Uploading Tracer Profile")
            try:
                """
                Unlike yum, the transaction is already written to the DB
                by this point so we don't need to do any work to give Tracer
                a list of affected apps.
                """
                upload_tracer_profile(query_affected_apps, self)
            except Exception:
                if (conf.has_option('main', 'supress_errors') and not
                   conf.getboolean('main', 'supress_errors')):
                    logger.error("Unable to upload Tracer Profile")



#!/usr/bin/python
#
# Copyright 2019 ATIX AG
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should

from os import path, environ
import logging
from katello.tracer import upload_tracer_profile
from katello.zypper_tracer import collect_apps
from zypp_plugin import Plugin


class TracerUploadPlugin(Plugin):
    def PLUGINEND(self, headers, body):
        logging.info("PLUGINEND")

        logging.info("Uploading Tracer Profile")
        try:
            upload_tracer_profile(collect_apps)
        except:
            logging.error("Unable to upload Tracer Profile")

        self.ack()

if __name__ == '__main__':
    if "DISABLE_TRACER_UPLOAD_PLUGIN" in environ:
        logging.info("$DISABLE_TRACER_UPLOAD_PLUGIN is set - disabling katello tracer upload plugin")

        # a dummy Plugin is needed
        plugin = Plugin()
    else:
        plugin = TracerUploadPlugin()

    plugin.main()



from __future__ import absolute_import
from os import path

# The file is created by /etc/kernel/postinst.d/unattended-upgrades (part of unattended-upgrades pkg)
REBOOT_NEEDED_FLAG = "/var/run/reboot-required"


class AptTracerApp:
    pass


def collect_apps(plugin=None):
    apps = []
    if path.isfile(REBOOT_NEEDED_FLAG):
        app = AptTracerApp()
        app.name = "kernel"
        app.helper = "You will have to reboot your computer"
        app.type = "static"
        apps.append(app)
    return apps



import errno
import os
import sys

from katello.constants import DISABLE_PACKAGE_PROFILE_VAR, PACKAGE_CACHE_FILE, PACKAGE_PROFILE_PLUGIN_CONF, PROFILE_CACHE_FILE
from katello.uep import get_manager, lookup_consumer_id
from katello.utils import combined_profiles_enabled, plugin_enabled


def upload_package_profile(force=False):
    if not plugin_enabled(PACKAGE_PROFILE_PLUGIN_CONF, DISABLE_PACKAGE_PROFILE_VAR, force):
        return

    consumer_id = lookup_consumer_id()
    if consumer_id is None:
        sys.stderr.write("Cannot upload package profile. Is this client registered?\n")
    else:
        get_manager().profilelib._do_update()


def purge_package_cache():
    file_to_remove = PACKAGE_CACHE_FILE
    if combined_profiles_enabled():
        file_to_remove = PROFILE_CACHE_FILE
    try:
        os.remove(file_to_remove)
    except OSError:
        error = sys.exc_info()[1]  # backward and forward compatible way to get the exception
        if error.errno is not errno.ENOENT:
            sys.stderr.write("Unable to clear the package cache")



from __future__ import absolute_import
from katello.uep import get_uep, lookup_consumer_id
import sys
import imp

def collect_apps():
    raise Exception("This is not the method you want to use. Normally, this method should be provided by apt_tracer or zypper_tracer")

try:
    imp.find_module('apt')
    from katello.apt_tracer import collect_apps
    apt = True
except ImportError as e:
    apt = False

try:
    imp.find_module('dnf')
    dnf = True
except ImportError:
    dnf = False

try:
    imp.find_module('yum')
    yum = True
except ImportError:
    yum = False

if yum or dnf:
    from tracer.query import Query

try:
    imp.find_module('zypp_plugin')
    from katello.zypper_tracer import collect_apps
    zypp = True
except ImportError:
    zypp = False


def upload_tracer_profile(queryfunc, plugin=None):
    uep = get_uep()
    consumer_id = lookup_consumer_id()
    if consumer_id is None:
        sys.stderr.write("Cannot upload tracer data, is this client registered?\n")
    else:
        method = '/consumers/%s/tracer' % uep.sanitize(consumer_id)
        data = {"traces": get_apps(queryfunc, plugin)}
        uep.conn.request_put(method, data)


def query_affected_apps(plugin=None):
    if yum or dnf:
        query = Query()
        return query.affected_applications().get()
    elif zypp or apt:
        return collect_apps()
    else:
        raise Exception("Couldn't detect package manager. Failed to query affected apps!")


def get_apps(queryfunc, plugin=None):
    """
    Return a array with nested arrays
    containing name, how to restart & app type
    for every package that needs restarting
    """
    apps = {}
    for app in queryfunc(plugin):
        apps[app.name] = {"helper": app.helper, "type": app.type}
    if plugin:
        # Don't report yum/dnf back if this if being ran via them.
        apps.pop("yum", None)
        apps.pop("dnf", None)
    return apps
